/********************************************************************
 * $Author: lindner $
 * $Revision: 1.1 $
 * $Date: 1992/12/10 23:13:27 $
 * $Source: /home/mudhoney/GopherSrc/release1.11/gopherd/RCS/openers.h,v $
 * $Status: $
 *
 * Paul Lindner, University of Minnesota CIS.
 *
 * Copyright 1991, 1992 by the Regents of the University of Minnesota
 * see the file "Copyright" in the distribution for conditions of use.
 *********************************************************************
 * MODULE: openers.c
 * Secure file access routines.
 *********************************************************************
 * Revision History:
 * $Log: openers.h,v $
 * Revision 1.1  1992/12/10  23:13:27  lindner
 * gopher 1.1 release
 *
 *
 *********************************************************************/

/*
 * Declarations for openers.c routines.
 */

/* restricted versions */
extern	int	ropen();
extern	FILE	*rfopen();
extern	int	rstat();
extern	DIR	*ropendir();
extern	int	rchdir();

/* unrestricted versions */
extern	int	uopen();
extern	FILE	*ufopen();
extern	int	ustat();
extern	DIR	*uopendir();
extern	int	uchdir();
