/*
 *  In this file you will find all of the global variables and 
 *  global defines.
 */


#define BOOLEAN      int
#define MAXGOPHERS   100
#define ROOT_LEVEL   0
#define ROOT_DIRECTORY "Root Directory"

#define MAXRESP      9

/*
 * This is some funky defines that assures that global variables are
 * declared only once.  (when globals.c includes this file with EXTERN
 * defined.
 */

#ifndef EXTERN
#define EXTERN extern
#define INIT(x)
#else
#define EXTERN
#define INIT(x) =x
#endif


/*** Global variables ***/

EXTERN char         PrinterCommand[80];
EXTERN char         PagerCommand[80];
EXTERN char         TelnetCommand[80];

EXTERN WINDOW       *MainWindow;
EXTERN GopherThing  OldGopher[MAXGOPHERS], Gopher[MAXGOPHERS];
EXTERN char         sGClearscreen[40];
EXTERN char         sGAudibleBell[8];
EXTERN char         **sGCSOFields;
EXTERN int          iGNumCSOFields;


/*** Externals ***/

extern char **environ;


/*** Prototypes and forward declarations ***/

void init_curses();
void display_file(/* char *Filename */);
void ZapCRLF();
int  outchar();
