/********************************************************************
 * $Author: lindner $
 * $Revision: 1.2 $
 * $Date: 1992/12/29 21:42:18 $
 * $Source: /home/mudhoney/GopherSrc/release1.11/object/RCS/STRstring.h,v $
 * $Status: $
 *
 * Paul Lindner, University of Minnesota CIS.
 *
 * Copyright 1991, 1992 by the Regents of the University of Minnesota
 * see the file "Copyright" in the distribution for conditions of use.
 *********************************************************************
 * MODULE: STRstring.h
 * A dynamic string in C that really wants to be C++
 *********************************************************************
 * Revision History:
 * $Log: STRstring.h,v $
 * Revision 1.2  1992/12/29  21:42:18  lindner
 * Added #defines for VMS (jqj,fogel)
 *
 * Revision 1.1  1992/12/10  23:27:52  lindner
 * gopher 1.1 release
 *
 *
 *********************************************************************/


#ifndef STRstring_H
#define STRstring_H

#ifdef VMS
/*** VMS linker doesn't know about case! ***/
#define STRcpy GSTRcpy
#define STRcmp GSTRcmp
#define STRcat GSTRcat
#endif

struct string_struct {
     int  len;
     char *data;
};

typedef struct string_struct String;

String *STRnew();
String *STRnewSet(/* char* */);
String *STRcpy(/* String, String */);
void    STRinit(/* String*  */);
void    STRset(/* String*, char* */);
void    STRdestroy(/* String* */);
#define STRget(s) ((s)->data)
#define STRlen(s) ((s)->len)
String* STRcat();
int     STRcmp();
#endif
