/********************************************************************
 * $Author: lindner $
 * $Revision: 1.2 $
 * $Date: 1992/12/29 21:44:47 $
 * $Source: /home/mudhoney/GopherSrc/release1.11/object/RCS/String.h,v $
 * $State: Rel $
 *
 * Paul Lindner, University of Minnesota CIS.
 *
 * Copyright 1991, 1992 by the Regents of the University of Minnesota
 * see the file "Copyright" in the distribution for conditions of use.
 *********************************************************************
 * MODULE: String.h
 * Header file that automagically includes string/strings.h
 *********************************************************************
 * Revision History:
 * $Log: String.h,v $
 * Revision 1.2  1992/12/29  21:44:47  lindner
 * Formatted it nice.  Added defines for VMS
 *
 * Revision 1.1  1992/12/10  23:27:52  lindner
 * gopher 1.1 release
 *
 *
 *********************************************************************/



/* string.h  == AT&T
   strings.h == BSD

Though your milage may vary... 

*/

#ifdef __convex__
#  include <stdarg.h>
#else
#  ifdef VMS
#    include string.h
#  else
#    include <string.h>
#  endif
#endif

#if defined(USG) || defined(hpux)
#  include <string.h>
#else
#  ifndef VMS
#    include <strings.h>
#   endif
#endif
