/*** All our mongo include files ***/

#include <stdio.h>

#ifndef IS_BSD
#include <stdlib.h>
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#ifndef IS_A_HPUX
#include <arpa/inet.h>
#endif

#include <netdb.h>
#include <signal.h>
#include <curses.h>
#include <string.h>

#ifndef IS_A_HPUX
#include <strings.h>
#endif

#include <ctype.h>

#define MAXSTR 128

#define A_FILE      '0'
#define A_DIRECTORY '1'
#define A_CSO       '2'
#define A_ERROR     '3'
#define A_BINHEX    '4'
#define A_TELNET    '5'
#define A_TECHMAIL  '6'
#define A_INDEX     '7'
#define A_EOI	    '.'

#define X_PREVIOUS   0
#define	X_SAVEFILE  -1
#define X_ABORT     -2
#define X_HELP      -3
#define X_OPTIONS   -4
/** Our Gopher Data Type **/

struct g_struct
{
     char    sFileType;       /* The type of thing it is */
     char    sTitle[MAXSTR]; /* User-displayed title of object       */
     char    sPath[MAXSTR];  /* Internal pathname to object on host */
     char    sHost[MAXSTR];  /* Internet name of host                */
     int     iPort;          /* Port number on host */
};

typedef struct g_struct GopherThing;


/** Get the configuration variables **/

#include "conf.h"
#include "globals.h"
