/*
 * These are all of user definable parameters
 */

/*
 * Make sure that your help file is in defined as $(LIBDIR)/gopher.hlp
 * where LIBDIR comes from the Makefile
 */

#define GOPHERHELP       "gopher.hlp"
#define DEFAULT_HOST     "gopher.micro.umn.edu"
#define GOPHER_PORT      150


/*
** Uncomment out the following if you plan on allowing anonymous logins
** to a gopher account.
*/

/*#define PARANOID /**/

/*
 * These are implementation specific #defines
 */

#ifdef IS_A_SUN
#define PAGER_COMMAND "/usr/ucb/more"
#define TELNET_COMMAND "/usr/ucb/telnet"
#define PRINTER_COMMAND "/usr/ucb/lpr"
#define USE_DIRENT
#endif

#ifdef IS_A_HPUX
#define SYSVCURSES
#define PAGER_COMMAND "/usr/bin/more"
#define TELNET_COMMAND "/usr/bin/telnet"
#define PRINTER_COMMAND "/usr/bin/lpr"
#define USE_DIRENT
#endif

#ifdef IS_AUX_BOX
#define SYSVCURSES
#define PAGER_COMMAND "/bin/more"
#define TELNET_COMMAND "/usr/bin/telnet"
#define PRINTER_COMMAND "/usr/ucb/lpr"
#endif

#ifdef IS_A_RS6000
#define SYSVCURSES
#define PAGER_COMMAND "/usr/ucb/more"
#define TELNET_COMMAND "/usr/ucb/telnet"
#define PRINTER_COMMAND "/bin/lpr"
#endif

#ifdef IS_BSD
#define PAGER_COMMAND "/usr/ucb/more"
#define TELNET_COMMAND "/usr/ucb/telnet"
#define PRINTER_COMMAND "/usr/ucb/lpr"
#define tolower
#define NO_VPRINTF
#endif


