/*
 *******************************************************************
 * $Author: lindner $
 * $Revision: 1.1 $
 * $Date: 91/03/25 21:11:32 $
 * $Source: /export/mermaid/staff/lindner/gopherd/RCS/conf.h,v $
 *******************************************************************
 */

/*
**  You'll find all the user configurable parameters here
*/


/*
** This specifies which port to run at.  We use 150.  Although gopher
** can run just about anywhere.
*/

#define GOPHER_PORT	7000

/*
** If your hostname command returns the Fully Qualified Domain Name
** (i.e. it looks like foo.bar.edu and not just foo) then make
** the domain name a null string.  Otherwise put in the rest of
** your domain name that `hostname` doesn't return.
*/

#define DOMAIN_NAME ".micro.umn.edu"

/*
** This is the place where all the data is going to reside.
*/

#define DATA_DIRECTORY "/home/mermaid/staff/lindner/gopher-data"


/*
** Uncomment this out if you are running a system that doesn't have
** dirent directory routines.  Note that Next (and Mach) have the
** dirent routines, but they don't seem to work.
*/

#define BROKENDIRS /**/
