/*
 *******************************************************************
 * $Author: $
 * $Revision: $
 * $Date: $
 * $Source: $
 *******************************************************************
 */

#include "conf.h"

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <pwd.h>
#include <errno.h>
#include <string.h>
#include <strings.h>
#include <sys/stat.h>
#include <time.h>

#ifndef BROKENDIRS
#  include <dirent.h>
#else
#  include <sys/dir.h>
#endif

typedef int boolean;
#define TRUE 1
#define FALSE 0

#include "globals.h"

char *pname;
